class CorpusAnalysisApp {
  constructor(corpusId) {
    this.data = {};

    // HTML elements
    this.elements = {
      container: document.querySelector('#corpus-analysis-app-container'),
      extensionTabs: document.querySelector('#corpus-analysis-app-extension-tabs'),
      initModal: document.querySelector('#corpus-analysis-app-init-modal'),
      overview: document.querySelector('#corpus-analysis-app-overview')
    };
    // Materialize elements
    this.elements.m = {
      extensionTabs: M.Tabs.init(this.elements.extensionTabs),
      initModal: M.Modal.init(this.elements.initModal, {dismissible: false})
    };

    this.extensions = {};

    this.settings = {
      corpusId: corpusId
    };
  }

  init() {
    this.disableActionElements();
    this.elements.m.initModal.open();
    // Init data
    this.data.cQiClient = new CQiClient(this.settings.corpusId);
    this.data.cQiClient.connect()
      .then(cQiStatus => {
        return this.data.cQiClient.corpora.get(`NOPAQUE_${this.settings.corpusId}`);
      })
      .then(
        cQiCorpus => {
          this.data.corpus = {o: cQiCorpus};
          this.data.corpus.o.getVisualizationData().then(data => console.log(data));
          // this.data.corpus.o.getVisualizationData()
          //   .then(
          //     (visualizationData) => {
          //       console.log(visualizationData);
          //       this.renderGeneralCorpusInfo(visualizationData);
          //       this.renderTextInfoList(visualizationData);
          //       this.renderTextProportionsGraphic(visualizationData);
          //     }
          //   );
          this.data.corpus.o.getCorpusData()
            .then(corpusData => {
              this.renderGeneralCorpusInfo(corpusData);
              this.renderTextInfoList(corpusData);
              this.renderTextProportionsGraphic(corpusData);
              this.renderWordFrequenciesGraphic(corpusData);
              this.renderWordDistributionsGraphic(corpusData);
            });
          // TODO: Don't do this hgere
          cQiCorpus.updateDb();
          this.enableActionElements();
          for (let extension of Object.values(this.extensions)) {extension.init();}
          this.elements.m.initModal.close();
        },
        cQiError => {
          let errorsElement = this.elements.initModal.querySelector('.errors');
          let progressElement = this.elements.initModal.querySelector('.progress');
          errorsElement.innerText = JSON.stringify(cQiError);
          errorsElement.classList.remove('hide');
          progressElement.classList.add('hide');
          if ('payload' in cQiError && 'code' in cQiError.payload && 'msg' in cQiError.payload) {
            app.flash(`${cQiError.payload.code}: ${cQiError.payload.msg}`, 'error');
          }
        }
      );

  
    // Add event listeners
    for (let extensionSelectorElement of this.elements.overview.querySelectorAll('.extension-selector')) {
      extensionSelectorElement.addEventListener('click', () => {
        this.elements.m.extensionTabs.select(extensionSelectorElement.dataset.target);
      });
    }
  }

  registerExtension(extension) {
    if (extension.name in this.extensions) {
      console.error(`Can't register extension ${extension.name}: Already registered`);
      return;
    }
    this.extensions[extension.name] = extension;
    if ('cQiClient' in this.data && this.data.cQiClient.connected) {extension.init();}
  }

  disableActionElements() {
    let actionElements = this.elements.container.querySelectorAll('.corpus-analysis-action');
    for (let actionElement of actionElements) {
      if (actionElement.nodeName === 'INPUT') {
        actionElement.disabled = true;
      } else if (actionElement.nodeName === 'SELECT') {
        actionElement.parentNode.querySelector('input.select-dropdown').disabled = true;
      } else {
        actionElement.classList.add('disabled');
      }
    }
  }

  enableActionElements() {
    let actionElements = this.elements.container.querySelectorAll('.corpus-analysis-action');
    for (let actionElement of actionElements) {
      if (actionElement.nodeName === 'INPUT') {
        actionElement.disabled = false;
      } else if (actionElement.nodeName === 'SELECT') {
        actionElement.parentNode.querySelector('input.select-dropdown').disabled = false;
      } else {
        actionElement.classList.remove('disabled');
      }
    }
  }

  renderGeneralCorpusInfo(corpusData) {
    document.querySelector('.corpus-num-tokens').innerHTML = corpusData.corpus.counts.token;
    document.querySelector('.corpus-num-s').innerHTML = corpusData.corpus.counts.s;
    // corpusGeneralInfoListElement.querySelector('.corpus-text-count').innerHTML = <b>Corpus text count:</b> ${Object.entries(corpusData.text).length;
    document.querySelector('.corpus-num-unique-words').innerHTML = Object.entries(corpusData.corpus.freqs.word).length;
    document.querySelector('.corpus-num-unique-lemmas').innerHTML = Object.entries(corpusData.corpus.freqs.lemma).length;
    document.querySelector('.corpus-num-unique-pos').innerHTML = Object.entries(corpusData.corpus.freqs.pos).length;
    document.querySelector('.corpus-num-unique-simple-pos').innerHTML = Object.entries(corpusData.corpus.freqs.simple_pos).length;
  }

  renderTextInfoList(corpusData) {
    // let corpusTextInfoListElement = document.querySelector('.corpus-text-info-list');
    // let corpusTextInfoList = new CorpusTextInfoList(corpusTextInfoListElement);
    // for (let text of Object.values(corpusData.text)) {
    //   text.values.title = corpusData.value_lookups.text.title[text.values.title];
    // }
    // corpusTextInfoList.add(Object.values(corpusData.text));

    // let textCountChipElement = document.querySelector('.text-count-chip');
    // textCountChipElement.innerHTML = `Text count: ${Object.values(corpusData.text).length}`;
  }

  renderTextProportionsGraphic(corpusData) {
    // let textProportionsGraphicElement = document.querySelector('#text-proportions-graphic');
    // let texts = Object.values(corpusData.text);
    // let graphData = [
    //   {
    //     values: texts.map(text => text.counts.token),
    //     labels: texts.map(text => `${text.values.title} (${text.values.publishing_year})`),
    //     type: 'pie'
    //   }
    // ];
    // let graphLayout = {
    //   height: 400,
    //   width: 500
    // };
    // Plotly.newPlot(textProportionsGraphicElement, graphData, graphLayout);
  }

  renderWordFrequenciesGraphic(corpusData) {
    // let wordFrequenciesGraphicElement = document.querySelector('#word-frequencies-graphic');
    // let words = Object.entries(corpusData.value_lookups.token.word);
    // let texts = Object.values(corpusData.text);
    // let graphData = [];
    // for (let word of words) {
    //   let data = {
    //     x: texts.map(text => `${text.values.title} (${text.values.publishing_year})`),
    //     y: texts.map(text => text.freqs.word[word[0]]),
    //     name: word[1],
    //     type: 'bar'
    //   };
    //   graphData.push(data);
    // }

    // let graphLayout = {
    //   height: 400,
    //   width: 500,
    //   barmode: 'stack',
    //   type: 'bar'
    // };
    // Plotly.newPlot(wordFrequenciesGraphicElement, graphData, graphLayout);
  }

  renderWordDistributionsGraphic(corpusData) {
    // let wordDistributionGraphicElement = document.querySelector('#word-distributions-graphic');
    // var trace1 = {
    //   x: [1, 2, 3, 4],
    //   y: [10, 11, 12, 13],
    //   mode: 'markers',
    //   marker: {
    //     size: [40, 60, 80, 100]
    //   }
    // };
    // var data = [trace1];
    // var layout = {
    //   title: 'Marker Size',
    //   showlegend: false,
    //   height: 600,
    //   width: 600
    // };
    // Plotly.newPlot(wordDistributionGraphicElement, data, layout);
  }
}
